@@ -1165,6 +1165,10 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 
 	src = mnt_fs_get_source(cxt->fs);
 
+	if (!src && (cxt->mountflags & MS_PROPAGATION))
+		/* mount --make-{shared,private,...} */
+		return mnt_fs_set_source(cxt->fs, "none");
+
 	/* ignore filesystems without source or filesystems
 	 * where the source is quasi-path (//foo/bar)
 	 */
@@ -1203,7 +1207,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 
 	if ((cxt->mountflags & (MS_BIND | MS_MOVE | MS_PROPAGATION)) ||
 	    mnt_fs_is_pseudofs(cxt->fs)) {
-		DBG(CXT, mnt_debug_h(cxt, "PROPAGATION/pseudo FS source: %s", path));
+		DBG(CXT, mnt_debug_h(cxt, "BIND/MOVE/pseudo FS source: %s", path));
 		return rc;
 	}
 
@@ -1259,7 +1263,7 @@ int mnt_context_guess_fstype(struct libmnt_context *cxt)
 {
 	char *type;
 	const char *dev;
-	int rc = -EINVAL;
+	int rc = 0;
 
 	assert(cxt);
 	assert(cxt->fs);
@@ -1286,7 +1290,7 @@ int mnt_context_guess_fstype(struct libmnt_context *cxt)
 
 	dev = mnt_fs_get_srcpath(cxt->fs);
 	if (!dev)
-		goto err;
+		goto done;
 
 	if (access(dev, F_OK) == 0) {
 		struct libmnt_cache *cache = mnt_context_get_cache(cxt);
@@ -1303,17 +1307,12 @@ int mnt_context_guess_fstype(struct libmnt_context *cxt)
 		else if (!strncmp(dev, "//", 2))
 			rc = mnt_fs_set_fstype(cxt->fs, "cifs");
 	}
-	if (rc)
-		goto err;
+
 done:
-	DBG(CXT, mnt_debug_h(cxt, "FS type: %s",
-				mnt_fs_get_fstype(cxt->fs)));
-	return 0;
+	DBG(CXT, mnt_debug_h(cxt, "FS type: %s", mnt_fs_get_fstype(cxt->fs)));
+	return rc;
 none:
 	return mnt_fs_set_fstype(cxt->fs, "none");
-err:
-	DBG(CXT, mnt_debug_h(cxt, "failed to detect FS type"));
-	return rc;
 }
 
 /*
