@@ -29,7 +29,6 @@
 /*
  * last
  */
-#include <err.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -51,6 +50,7 @@
 #include "pathnames.h"
 #include "nls.h"
 #include "xalloc.h"
+#include "c.h"
 
 #define	SECDAY	(24*60*60)			/* seconds in a day */
 #define	NO	0				/* false/no */
@@ -107,8 +107,6 @@ static char *ttyconv(char *);
 
 int
 main(int argc, char **argv) {
-	extern int	optind;
-	extern char	*optarg;
 	int	ch;
 
 	setlocale(LC_ALL, "");
@@ -366,10 +364,12 @@ want(struct utmp *bp, int check) {
 				return YES;
 			break;
 		case INET_TYPE:
-			if (bp->ut_addr == inet_addr(step->name))
+			if ((in_addr_t) bp->ut_addr == inet_addr(step->name))
 			  return YES;
 			break;
-	}
+		default:
+			abort();
+		}
 	return NO;
 }
 
@@ -444,11 +444,11 @@ ttyconv(char *arg) {
 	if (strlen(arg) == 2) {
 		/* either 6 for "ttyxx" or 8 for "console" */
 		mval = xmalloc(8);
-		if (!strcmp(arg, "co"))
+		if (!strncmp(arg, "co", 2))
 			(void)strcpy(mval, "console");
 		else {
 			(void)strcpy(mval, "tty");
-			(void)strcpy(mval + 3, arg);
+			(void)strncpy(mval + 3, arg, 4);
 		}
 		return mval;
 	}
