@@ -43,46 +43,67 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
-#include <strings.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <getopt.h>
+
+#include "c.h"
 #include "nls.h"
+#include "strutils.h"
 
 #define	SYSLOG_NAMES
 #include <syslog.h>
 
-int	decode __P((char *, CODE *));
-int	pencode __P((char *));
-void	usage __P((void));
+int	decode (char *, CODE *);
+int	pencode (char *);
 
 static int optd = 0;
+static int udpport = 514;
 
 static int
 myopenlog(const char *sock) {
        int fd;
        static struct sockaddr_un s_addr; /* AF_UNIX address of local logger */
 
-       if (strlen(sock) >= sizeof(s_addr.sun_path)) {
-	       printf (_("logger: openlog: pathname too long\n"));
-	       exit(1);
-       }
+       if (strlen(sock) >= sizeof(s_addr.sun_path))
+	       errx(EXIT_FAILURE, _("openlog %s: pathname too long"), sock);
 
        s_addr.sun_family = AF_UNIX;
        (void)strcpy(s_addr.sun_path, sock);
 
-       if ((fd = socket(AF_UNIX, optd ? SOCK_DGRAM : SOCK_STREAM, 0)) == -1) {
-               printf (_("socket: %s.\n"), strerror(errno));
-               exit (1);
-       }
+       if ((fd = socket(AF_UNIX, optd ? SOCK_DGRAM : SOCK_STREAM, 0)) == -1)
+	       err(EXIT_FAILURE, _("socket %s"), sock);
+
+       if (connect(fd, (struct sockaddr *) &s_addr, sizeof(s_addr)) == -1)
+	       err(EXIT_FAILURE, _("connect %s"), sock);
 
-       if (connect(fd, (struct sockaddr *) &s_addr, sizeof(s_addr)) == -1) {
-               printf (_("connect: %s.\n"), strerror(errno));
-               exit (1);
-       }
        return fd;
 }
 
+static int
+udpopenlog(const char *servername,int port) {
+	int fd;
+	struct sockaddr_in s_addr;
+	struct hostent *serverhost;
+
+	if ((serverhost = gethostbyname(servername)) == NULL )
+		errx(EXIT_FAILURE, _("unable to resolve '%s'"), servername);
+
+	if ((fd = socket(AF_INET, SOCK_DGRAM , 0)) == -1)
+		err(EXIT_FAILURE, _("socket"));
+
+	bcopy(serverhost->h_addr,&s_addr.sin_addr,serverhost->h_length);
+        s_addr.sin_family=AF_INET;
+        s_addr.sin_port=htons(port);
+
+        if (connect(fd, (struct sockaddr *) &s_addr, sizeof(s_addr)) == -1)
+		err(EXIT_FAILURE, _("connect"));
+
+	return fd;
+}
 static void
 mysyslog(int fd, int logflags, int pri, char *tag, char *msg) {
        char buf[1000], pid[30], *cp, *tp;
@@ -111,6 +132,28 @@ mysyslog(int fd, int logflags, int pri, char *tag, char *msg) {
        }
 }
 
+static void __attribute__ ((__noreturn__)) usage(FILE *out)
+{
+	fputs(_("\nUsage:\n"), out);
+	fprintf(out,
+	      _(" %s [options] [message]\n"), program_invocation_short_name);
+
+	fputs(_("\nOptions:\n"), out);
+	fputs(_(" -d, --udp             use UDP (TCP is default)\n"
+		" -i, --id              log the process ID too\n"
+		" -f, --file <file>     log the contents of this file\n"
+		" -h, --help            display this help text and exit\n"), out);
+	fputs(_(" -n, --server <name>   write to this remote syslog server\n"
+		" -P, --port <number>   use this UDP port\n"
+		" -p, --priority <prio> mark given message with this priority\n"
+		" -s, --stderr          output message to standard error as well\n"), out);
+	fputs(_(" -t, --tag <tag>       mark every line with this tag\n"
+		" -u, --socket <socket> write to this Unix socket\n"
+		" -V, --version         output version information and exit\n\n"), out);
+
+	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+}
+
 /*
  * logger -- read and log utility
  *
@@ -122,7 +165,24 @@ main(int argc, char **argv) {
 	int ch, logflags, pri;
 	char *tag, buf[1024];
 	char *usock = NULL;
+	char *udpserver = NULL;
 	int LogSock = -1;
+	long tmpport;
+
+	static const struct option longopts[] = {
+		{ "id",		no_argument,	    0, 'i' },
+		{ "stderr",	no_argument,	    0, 's' },
+		{ "file",	required_argument,  0, 'f' },
+		{ "priority",	required_argument,  0, 'p' },
+		{ "tag",	required_argument,  0, 't' },
+		{ "socket",	required_argument,  0, 'u' },
+		{ "udp",	no_argument,	    0, 'd' },
+		{ "server",	required_argument,  0, 'n' },
+		{ "port",	required_argument,  0, 'P' },
+		{ "version",	no_argument,	    0, 'V' },
+		{ "help",	no_argument,	    0, 'h' },
+		{ NULL,		0, 0, 0 }
+	};
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -131,15 +191,13 @@ main(int argc, char **argv) {
 	tag = NULL;
 	pri = LOG_NOTICE;
 	logflags = 0;
-	while ((ch = getopt(argc, argv, "f:ip:st:u:d")) != -1)
+	while ((ch = getopt_long(argc, argv, "f:ip:st:u:dn:P:Vh",
+					    longopts, NULL)) != -1) {
 		switch((char)ch) {
 		case 'f':		/* file to log */
-			if (freopen(optarg, "r", stdin) == NULL) {
-				int errsv = errno;
-				(void)fprintf(stderr, _("logger: %s: %s.\n"),
-				    optarg, strerror(errsv));
-				exit(1);
-			}
+			if (freopen(optarg, "r", stdin) == NULL)
+				err(EXIT_FAILURE, _("file %s"),
+				    optarg);
 			break;
 		case 'i':		/* log process id also */
 			logflags |= LOG_PID;
@@ -159,16 +217,37 @@ main(int argc, char **argv) {
 		case 'd':
 			optd = 1;	/* use datagrams */
 			break;
+		case 'n':		/* udp socket */
+			optd = 1;	/* use datagrams because udp */
+			udpserver = optarg;
+			break;
+		case 'P':		/* change udp port */
+			tmpport = strtol_or_err(optarg,
+						_("failed to parse port number"));
+			if (tmpport < 0 || 65535 < tmpport)
+				errx(EXIT_FAILURE, _("port `%ld' out of range"),
+						tmpport);
+			udpport = (int) tmpport;
+			break;
+		case 'V':
+			printf(_("%s from %s\n"), program_invocation_short_name,
+						  PACKAGE_STRING);
+			exit(EXIT_SUCCESS);
+		case 'h':
+			usage(stdout);
 		case '?':
 		default:
-			usage();
+			usage(stderr);
 		}
+	}
 	argc -= optind;
 	argv += optind;
 
 	/* setup for logging */
-	if (!usock)
+	if (!usock && !udpserver)
 		openlog(tag ? tag : getlogin(), logflags, 0);
+	else if (udpserver)
+		LogSock = udpopenlog(udpserver,udpport);
 	else
 		LogSock = myopenlog(usock);
 
@@ -177,19 +256,19 @@ main(int argc, char **argv) {
 	/* log input line if appropriate */
 	if (argc > 0) {
 		register char *p, *endp;
-		int len;
+		size_t len;
 
 		for (p = buf, endp = buf + sizeof(buf) - 2; *argv;) {
 			len = strlen(*argv);
 			if (p + len > endp && p > buf) {
-			    if (!usock)
+			    if (!usock && !udpserver)
 				syslog(pri, "%s", buf);
 			    else
 				mysyslog(LogSock, logflags, pri, tag, buf);
 				p = buf;
 			}
 			if (len > sizeof(buf) - 1) {
-			    if (!usock)
+			    if (!usock && !udpserver)
 				syslog(pri, "%s", *argv++);
 			    else
 				mysyslog(LogSock, logflags, pri, tag, *argv++);
@@ -201,12 +280,12 @@ main(int argc, char **argv) {
 			}
 		}
 		if (p != buf) {
-		    if (!usock)
+		    if (!usock && !udpserver)
 			syslog(pri, "%s", buf);
 		    else
 			mysyslog(LogSock, logflags, pri, tag, buf);
 		}
-	} else
+	} else {
 		while (fgets(buf, sizeof(buf), stdin) != NULL) {
 		    /* glibc is buggy and adds an additional newline,
 		       so we have to remove it here until glibc is fixed */
@@ -215,24 +294,25 @@ main(int argc, char **argv) {
 		    if (len > 0 && buf[len - 1] == '\n')
 			    buf[len - 1] = '\0';
 
-		    if (!usock)
+		    if (!usock && !udpserver)
 			syslog(pri, "%s", buf);
 		    else
 			mysyslog(LogSock, logflags, pri, tag, buf);
 		}
-	if (!usock)
+	}
+	if (!usock && !udpserver)
 		closelog();
 	else
 		close(LogSock);
-	exit(0);
+
+	return EXIT_SUCCESS;
 }
 
 /*
  *  Decode a symbolic name to a numeric value
  */
 int
-pencode(s)
-	register char *s;
+pencode(char *s)
 {
 	char *save;
 	int fac, lev;
@@ -241,11 +321,9 @@ pencode(s)
 	if (*s) {
 		*s = '\0';
 		fac = decode(save, facilitynames);
-		if (fac < 0) {
-			(void)fprintf(stderr,
-			    _("logger: unknown facility name: %s.\n"), save);
-			exit(1);
-		}
+		if (fac < 0)
+			errx(EXIT_FAILURE,
+			    _("unknown facility name: %s."), save);
 		*s++ = '.';
 	}
 	else {
@@ -253,18 +331,14 @@ pencode(s)
 		s = save;
 	}
 	lev = decode(s, prioritynames);
-	if (lev < 0) {
-		(void)fprintf(stderr,
-		    _("logger: unknown priority name: %s.\n"), save);
-		exit(1);
-	}
+	if (lev < 0)
+		errx(EXIT_FAILURE,
+		    _("unknown priority name: %s."), save);
 	return ((lev & LOG_PRIMASK) | (fac & LOG_FACMASK));
 }
 
 int
-decode(name, codetab)
-	char *name;
-	CODE *codetab;
+decode(char *name, CODE *codetab)
 {
 	register CODE *c;
 
@@ -277,11 +351,3 @@ decode(name, codetab)
 
 	return (-1);
 }
-
-void
-usage()
-{
-	(void)fprintf(stderr,
-	    _("usage: logger [-is] [-f file] [-p pri] [-t tag] [-u socket] [ message ... ]\n"));
-	exit(1);
-}
